/*******************************************************************************
 * Copyright 2006  Hon Hai Precision Ind. Co. Ltd.
 * All Rights Reserved.
 * No portions of this material shall be reproduced in any form without the
 * written permission of Hon Hai Precision Ind. Co. Ltd.
 *
 * All information contained in this document is Hon Hai Precision Ind.
 * Co. Ltd. company private, proprietary, and trade secret property and
 * are protected by international intellectual property laws and treaties.
 *
 ******************************************************************************/

/*******************************************************************************
 ***  Filename:
 ***      cmd_board.c
 ***
 ***  Description:
 ***      uboot commands for board-specific utilities
 ***
 ***  History:
 ***
 ***   Modify Reason                    Author          Date            Search Flag(Option)
 ***--------------------------------------------------------------------------------------
 ***   Created                          Peter Ling      20006/11/23
 ***
 ******************************************************************************/

#include <common.h>
#include <command.h>
#include <net.h>

#include "mvGpp.h"
#include "board_config.h"
#include "lzma.h"

int 
do_chk_flash_image (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    char command[100];

    /* 
     * If flash image checksum error,
     * or failed to decode LZMA kernel, 
     * then run TFTP server until we get a valid image.
     */
    if (verify_flash_image() ||
        decode_flash_image())
    {
        sprintf(command, "tftpd 0x%lX", IMAGE_DL_ADDR);
        run_command(command, 0);
        
        while (verify_download_image(IMAGE_DL_ADDR))
        {
            /* Something wrong with this image, run TFTP server again */
            sprintf(command, "tftpd 0x%lX", IMAGE_DL_ADDR);
            run_command(command, 0);
        }

        /* Image is OK, write to flash and reboot */
        write_image_to_flash(IMAGE_DL_ADDR);
        printf("Image update complete! Reboot now\n");
        run_command("reset", 0);
        while(1);
    }

    return 0;    
}

U_BOOT_CMD(
    chkFlashImage, 1, 1, do_chk_flash_image,
    "chkFlashImage - verify checksums on flash images\n",
    NULL
);


/*
 * Test command for LZMA decode
 */
int 
do_lzma_decode (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned long src, dst;
    unsigned char *src_addr, *dst_addr;
    unsigned int  src_len, dst_len;

    if (argc != 4)
    {
        printf("Usage: %s <src addr> <dst addr> <src len>\n", argv[0]);
        return 1;
    }

    src = simple_strtol(argv[1], NULL, 16);
    dst = simple_strtol(argv[2], NULL, 16);

    src_addr = (unsigned char *)src;    
    dst_addr = (unsigned char *)dst;
    src_len = simple_strtol(argv[3], NULL, 16);

    printf("Decompressing image from 0x%lX to 0x%lX...\n", src_addr, dst_addr);

    return lzma_decode(src_addr, dst_addr, src_len, &dst_len);
}

U_BOOT_CMD(
	lzmadecode,	4,		1,	do_lzma_decode,
 	"lzmadecode - decode LZMA image\n",
	NULL
);

/* 
 * Test command for GPIO
 */
int 
do_gpio (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned int  gpio_num, gpio_on_off;

    if (argc != 3)
    {
        printf("Usage: %s <gpio num> <1|0>\n", argv[0]);
        return 1;
    }

    gpio_num    = simple_strtol(argv[1], NULL, 16);
    gpio_on_off = simple_strtol(argv[2], NULL, 16);

    if (gpio_on_off)
        mvGppValueSet(0, 1L << gpio_num, 1L << gpio_num);
    else
        mvGppValueSet(0, 1L << gpio_num, 0);
    return 0;
}

U_BOOT_CMD(
	gpio,	3,		1,	do_gpio,
 	"gpio - test gpio\n",
	NULL
);


int 
do_chk_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    return check_reset();
}

U_BOOT_CMD(
	chkReset,	3,		1,	do_chk_reset,
 	"chkReset - check reset button\n",
	NULL
);
